{{
  This program first checks for a byte received from the handbox controller
  If a byte is present, the slew rate is extracted from it (a value from 0 to 3)
  If the byte was sent by the North or South button being pressed,
    the declination motor is driven in the appropriate direction at a speed of slew_rate x sidereal_rate
  If no byte was received, or the received byte doesn't pertain to the declination motor, then the autoguider is checked
    If the autoguider directs movement North or South, the declination motor is driven in the appropriate direction at 1x sidereal rate
  Otherwise, the declination motor remains unpowered.

  If a received byte was sent by the West or East button being pressed,
    the right-ascension motor is driven in the appropriate direction at a speed of slew_rate x sidereal_rate + sidereal rate
  If no byte was received, or the received byte doesn't pertain to the R.A. motor, then the autoguider is checked
    If the autoguider directs movement West or East, the R.A. motor is driven in the appropriate direction at 1x sidereal rate + sidereal rate
  Otherwise, the R.A. motor is driven Westward at 1x sidereal rate
}}
CON  ' DON'T TOUCH!
  _xinfreq      = 5_000_000                             ' using 5MHz xtal 
  _clkmode      = xtal1 + pll16x                        

  RA   = 0
  DEC  = 1
  NORTH = %0001
  SOUTH = %0010
  WEST  = %0100
  EAST  = %1000

CON  ' Modify as needed
'PIN ASSIGNMENTS
  AUTO_N    = 24                ' Autoguider North                                                              
  AUTO_S    = 25                ' Autoguider South                                                              
  AUTO_W    = 26                ' Autoguider West                                                               
  AUTO_E    = 27                ' Autoguider East                                                               
  RA_SPEED  = 16                ' RA motor enable                                                               
  RA_DIR    = 17                ' RA motor phase                                                                
  DEC_SPEED = 18                ' Dec motor enable                                                              
  DEC_DIR   = 19                ' Dec motor phase                                                               
  TX        = 10                ' Transmit to handbox                                                           
  RX        = 11                ' Receive from handbox                                                          

  PWM_FREQ  = 50                ' 50Hz PWM frequency = 20ms pulse width                  
  SIDE_RATE = 7_5               ' Sidereal tracking rate 7.5% = 1.5ms out of 20ms pulse width                   
                                ' Adjust as needed to counteract Earth's rotation
  AUTO_RATE = 7_5               ' Autoguider correction rate - currently set to same 7.5% value as the sidereal rate
                                
  SLEW_1    = 2                 ' Slowest slew rate from handbox.  These values are multiplied by the sidereal rate
  SLEW_2    = 4
  SLEW_3    = 6
  SLEW_4    = 8                 ' Fastest slew rate from handbox
                                                        
VAR
  long  slew_rate
  long  rx_byte
  long  debug_stack[20]
  
OBJ
  motor : "dual_motor_ez_MOD"   ' dual motor driver
  ser   : "FullDuplexSerial"    ' used for communicating with the handbox
  fds   : "FullDuplexSerial"    ' used for debugging

PUB Main | t
  motor.start(-1, RA_SPEED, RA_DIR, -1, DEC_SPEED, DEC_DIR, PWM_FREQ)           ' Start the motor driver and assign pins, 50Hz PWM frequency
  ser.start(RX,TX,0,9600)
  cognew(debug,@debug_stack)

  t := clkfreq / PWM_FREQ + cnt
  repeat
    waitcnt(t += clkfreq / PWM_FREQ)                                            ' Repeat 50 times per second (matches PWM frequency)
    rx_byte := ser.rxcheck #> 0                                                 ' Read a byte from the handbox
    if rx_byte > 0
      slew_rate := lookupz(rx_byte >> 4:SLEW_1,SLEW_2,SLEW_3,SLEW_4)            ' read the slew rate from the upper nibble 

    if rx_byte & NORTH                                                          ' NORTH handbox button pressed
      motor.set_speed(DEC,SIDE_RATE * slew_rate)                                ' Set the DEC motor speed to new rate
    elseif rx_byte & SOUTH                                                      ' SOUTH handbox button pressed
      motor.set_speed(DEC,-SIDE_RATE * slew_rate)
    elseif ina[AUTO_N]                                                          ' If the handbox is not controlling the DEC motor, check the autoguider
      motor.set_speed(DEC,AUTO_RATE)                                            
    elseif ina[AUTO_S]
      motor.set_speed(DEC,-AUTO_RATE)                                           
    else
      motor.set_speed(DEC,0)                                                    ' If neither the handbox nor autoguider is controlling the DEC motor, turn it off
  
    if rx_byte & WEST                                                           ' WEST handbox button pressed
      motor.set_speed(RA,SIDE_RATE * slew_rate + SIDE_RATE)                     ' Set the RA motor speed to new rate
    elseif rx_byte & EAST                                                       ' EAST handbox button pressed
      motor.set_speed(RA,-SIDE_RATE * slew_rate + SIDE_RATE)
    elseif ina[AUTO_W]                                                          ' If the handbox is not controlling the RA motor, check the autoguider
      motor.set_speed(RA,AUTO_RATE + SIDE_RATE)                                 '  slew West, with the sidereal rate
    elseif ina[AUTO_E]
      motor.set_speed(RA,-AUTO_RATE + SIDE_RATE)                                '  slew East, against the sidereal rate
    else
      motor.set_speed(RA,SIDE_RATE)                                             ' If neither the handbox nor autoguider is controlling the RA motor, set to SIDE_RATE

PRI debug
  fds.start(31,30,0,9600)
  repeat
    if rx_byte > 0
      if rx_byte & NORTH                     
        fds.str(string("North "))            
      elseif rx_byte & SOUTH                 
        fds.str(string("South "))            
      if rx_byte & WEST                      
        fds.str(string("West "))             
      elseif rx_byte & EAST                  
        fds.str(string("East "))             
      fds.dec(rx_byte >> 4)                  
      fds.tx($0D)                            
  